/*==============================================================================
 The Fraclet Programming Model - Fractal Component Model (fractal@objectweb.org)
 Copyright (C) 2002-2006 INRIA Futurs / LIFL
 Copyright (C) 2006-2007 Universitetet i Oslo

 This library is free software; you can redistribute it and/or modify it under 
 the terms of the GNU Lesser General Public License as published by the Free 
 Software Foundation; either version 2.1 of the License, or any later version.

 This library is distributed in the hope that it will be useful, but WITHOUT ANY 
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License along 
 with this library; if not, write to the Free Software Foundation, Inc., 
 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

 Initial developer(s): Romain Rouvoy
 Contributor(s): 
 =============================================================================*/
package org.objectweb.fractal.helloworld;

import static java.util.logging.Logger.getLogger;
import static org.objectweb.fractal.fraclet.types.Step.CREATE;

import java.util.logging.Logger;

import org.objectweb.fractal.api.control.NameController;
import org.objectweb.fractal.fraclet.annotations.Component;
import org.objectweb.fractal.fraclet.annotations.Controller;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.fraclet.types.Policy;
import org.objectweb.fractal.fraclet.types.Step;

/**
 * Implementation of a Helloworld client component.
 * 
 * @author Romain Rouvoy (Initial Developer)
 * @version $Revision$
 */
@Component(name = "helloworld.Client")
public class Client extends AbstractClient {
    private final Logger log = getLogger("client");
    private @Requires(name = "s", bind="bindService")
    Service service;
    protected @Controller("name-controller")
    NameController nc;

    @Lifecycle(step=Step.START,policy=Policy.ASYNCHRONOUS)
    protected void start() {
        log.info("helloworld.Client - started.");
    }

    @Lifecycle(step=Step.DESTROY)
    protected void stop() {
        log.info("helloworld.Client - stopped.");
    }
    
    protected void bindService(Service ref) {
        log.info("helloworld.Client - service bound.");
        this.service = ref;
    }

    /* (non-Javadoc)
     * @see java.lang.Runnable#run()
     */
    public void run() {
        try {
            this.service.print(this.nc.getFcName() + " says " + this.message);
        } catch (NullPointerException e) { // NPE when using the JavaBackend
            this.service.print("Client says " + this.message);
        }
    }
}